#ifdef __sun__
#include "/usr/ucbinclude/fcntl.h"
#include "b.hh"
extern "C" int gethostname(char*,int);
#endif

#include <stdlib.h>
#include <unistd.h>
#include <iostream.h>
#include <fcntl.h>
#include <string.h> 
#include <errno.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/param.h>

#include "debug.hh"

#ifdef __osf__
#include "net.h"
//extern "C" int gethostname(const char *,int);
#endif

#include "list.hh"
#include "sys_socket.hh"


/************************************************************************
**                              SYS_Address                            **
************************************************************************/


SYS_Address::SYS_Address(Port& _port)
{
  char  sName[MAXHOSTNAMELEN];// = "localhost";
  gethostname(sName,MAXHOSTNAMELEN);
  
  Initialize(sName,_port.port);
}


SYS_Address::SYS_Address(Transport_Address& _TAddr)
{
  Initialize((const char*)((String)(_TAddr.host)),(unsigned short)(_TAddr.port));
}



/******************** LINUX version ***************************/
#ifdef __linux__
void SYS_Address::Initialize(const char *sName,unsigned short port)
{
  hostent  *pHostent;
  bzero((char *)&sa, sizeof(sockaddr_in));
  if((pHostent = gethostbyname(sName)) != 0)
    {
      sa.sin_family      = AF_INET;
      bcopy((void *) pHostent->h_addr_list[0],
	    (void *) &(sa.sin_addr.s_addr),
	    pHostent->h_length);
      sa.sin_port               = htons(port);
      status = SYS_SOCKET_ADDRESS_STATUS_SET;
    }
  else
    {
      status = SYS_SOCKET_ADDRESS_STATUS_NOT_SET;
      cerr << "SYS_SOCKET: Request address not found\n";
    }
}
#endif



/******************** OSF version ***************************/
#ifdef __osf__
void SYS_Address::Initialize(const char *sName,unsigned short port)
{
  cout << "SYS_ADDRESS: Initialize: " << sName << ':' << port << '\n';
  hostent  *pHostent;
  bzero((char *)&sa, sizeof(sockaddr_in));

  if(isdigit(sName[0]))
    {
      cout << "SYS_ADDRESS: Search host bye address\n";
      unsigned int    addr = inet_addr(sName);
      pHostent = gethostbyaddr((char*)&addr,4,AF_INET);
    }
  else
    {
      cout << "SYS_ADDRESS: Search host bye name\n";
      pHostent = gethostbyname(sName);      
    }


  if(pHostent != 0)
    {
      sa.sin_family      = AF_INET;
      bcopy((char *) pHostent->h_addr_list[0],
	    (char *) &(sa.sin_addr.s_addr),
	    pHostent->h_length);
      sa.sin_port       = htons(port);
      cerr << "HOST_FOUND\n";
      status = SYS_SOCKET_ADDRESS_STATUS_SET;
    }
  else
    {
      cerr << "HOST_NOT_FOUND\n";
      status = SYS_SOCKET_ADDRESS_STATUS_NOT_SET;
    }
}
#endif


/****************** Solaris version *************************/
#ifdef __sun__
void SYS_Address::Initialize(const char *sName,unsigned short port)
{
  hostent  *pHostent;
  bzero((char *)&sa, sizeof(sockaddr_in));
  if((pHostent = gethostbyname(sName)) != 0)
    {
      sa.sin_family      = AF_INET;
      bcopy((char *) pHostent->h_addr_list[0],
	    (char *) &(sa.sin_addr.s_addr),
	    pHostent->h_length);
      sa.sin_port       = htons(port);
      status = SYS_SOCKET_ADDRESS_STATUS_SET;
    }
  else
    {
      cerr << "HOST_NOT_FOUND\n";
      status = SYS_SOCKET_ADDRESS_STATUS_NOT_SET;
    }
}
#endif


SYS_Address::~SYS_Address()
{
  ;
}

int SYS_Address::Status()
{
  return status;
}

